@@ -1,8 +1,7 @@
-use async_std::io::{
-    prelude::{ReadExt, WriteExt},
-    Read, Write,
-};
 use std::io;
+use std::ops::{Deref, DerefMut};
+
+use crate::runtime::{AsyncRead, AsyncReadExt, AsyncWrite, AsyncWriteExt};
 
 const RBUF_SIZE: usize = 8 * 1024;
 
@@ -23,7 +22,7 @@ pub struct BufStream<S> {
 
 impl<S> BufStream<S>
 where
-    S: Read + Write + Unpin,
+    S: AsyncRead + AsyncWrite + Unpin,
 {
     pub fn new(stream: S) -> Self {
         Self {
@@ -51,6 +50,12 @@ where
         Ok(())
     }
 
+    pub fn clear_bufs(&mut self) {
+        self.rbuf_rindex = 0;
+        self.rbuf_windex = 0;
+        self.wbuf.clear();
+    }
+
     #[inline]
     pub fn consume(&mut self, cnt: usize) {
         self.rbuf_rindex += cnt;
@@ -118,6 +123,20 @@ where
     }
 }
 
+impl<S> Deref for BufStream<S> {
+    type Target = S;
+
+    fn deref(&self) -> &Self::Target {
+        &self.stream
+    }
+}
+
+impl<S> DerefMut for BufStream<S> {
+    fn deref_mut(&mut self) -> &mut Self::Target {
+        &mut self.stream
+    }
+}
+
 // TODO: Find a nicer way to do this
 // Return `Ok(None)` immediately from a function if the wrapped value is `None`
 #[allow(unused)]
