@@ -16,11 +16,18 @@ mod database;
 mod executor;
 mod query;
 mod query_as;
+mod transaction;
 mod url;
 
+#[doc(hidden)]
+pub mod runtime;
+
 #[macro_use]
 pub mod arguments;
+
+#[doc(hidden)]
 pub mod decode;
+
 pub mod describe;
 pub mod encode;
 pub mod pool;
@@ -40,10 +47,11 @@ pub use database::Database;
 #[doc(inline)]
 pub use error::{Error, Result};
 
-pub use connection::Connection;
+pub use connection::{Connect, Connection};
 pub use executor::Executor;
 pub use query::{query, Query};
 pub use query_as::{query_as, QueryAs};
+pub use transaction::Transaction;
 
 #[doc(hidden)]
 pub use query_as::query_as_mapped;
