@@ -1,18 +1,18 @@
 use std::convert::TryInto;
 use std::io;
 
-use async_std::net::{Shutdown, TcpStream};
 use byteorder::{ByteOrder, LittleEndian};
 use futures_core::future::BoxFuture;
 use sha1::Sha1;
+use std::net::Shutdown;
 
 use crate::cache::StatementCache;
-use crate::connection::Connection;
-use crate::io::{Buf, BufMut, BufStream};
+use crate::connection::{Connect, Connection};
+use crate::io::{Buf, BufMut, BufStream, MaybeTlsStream};
 use crate::mysql::error::MySqlError;
 use crate::mysql::protocol::{
     AuthPlugin, AuthSwitch, Capabilities, Decode, Encode, EofPacket, ErrPacket, Handshake,
-    HandshakeResponse, OkPacket,
+    HandshakeResponse, OkPacket, SslRequest,
 };
 use crate::mysql::rsa;
 use crate::mysql::util::xor_eq;
@@ -28,8 +28,64 @@ const COLLATE_UTF8MB4_UNICODE_CI: u8 = 224;
 /// The connection string expected by [Connection::open] should be a MySQL connection
 /// string, as documented at
 /// <https://dev.mysql.com/doc/refman/8.0/en/connecting-using-uri-or-key-value-pairs.html#connecting-using-uri>
+///
+/// ### TLS Support (requires `tls` feature)
+/// This connection type supports some of the same flags as the `mysql` CLI application for SSL
+/// connections, but they must be specified via the query segment of the connection string
+/// rather than as program arguments.
+///
+/// The same options for `--ssl-mode` are supported as the `ssl-mode` query parameter:
+/// <https://dev.mysql.com/doc/refman/8.0/en/connection-options.html#option_general_ssl-mode>
+///
+/// ```text
+/// mysql://<user>[:<password>]@<host>[:<port>]/<database>[?ssl-mode=<ssl-mode>[&ssl-ca=<path>]]
+/// ```
+/// where
+/// ```text
+/// ssl-mode = DISABLED | PREFERRED | REQUIRED | VERIFY_CA | VERIFY_IDENTITY
+/// path = percent (URL) encoded path on the local machine
+/// ```
+///
+/// If the `tls` feature is not enabled, `ssl-mode=DISABLED` and `ssl-mode=PREFERRED` are no-ops and
+/// `ssl-mode=REQUIRED`, `ssl-mode=VERIFY_CA` and `ssl-mode=VERIFY_IDENTITY` are forbidden
+/// (attempting to connect with these will return an error).
+///
+/// If the `tls` feature is enabled, an upgrade to TLS is attempted on every connection by default
+/// (equivalent to `ssl-mode=PREFERRED`). If the server does not support TLS (because `--ssl=0` was
+/// passed to the server or an invalid certificate or key was used:
+/// <https://dev.mysql.com/doc/refman/8.0/en/using-encrypted-connections.html>)
+/// then it falls back to an unsecured connection and logs a warning.
+///
+/// Add `ssl-mode=REQUIRED` to your connection string to emit an error if the TLS upgrade fails.
+///
+/// However, like with `mysql` the server certificate is **not** checked for validity by default.
+///
+/// Specifying `ssl-mode=VERIFY_CA` will cause the TLS upgrade to verify the server's SSL
+/// certificate against a local CA root certificate; this is not the system root certificate
+/// but is instead expected to be specified as a local path with the `ssl-ca` query parameter
+/// (percent-encoded so the URL remains valid).
+///
+/// If you're running MySQL locally it might look something like this (for `VERIFY_CA`):
+/// ```text
+/// mysql://root:password@localhost/my_database?ssl-mode=VERIFY_CA&ssl-ca=%2Fvar%2Flib%2Fmysql%2Fca.pem
+/// ```
+///
+/// `%2F` is the percent-encoding for forward slash (`/`). In the example we give `/var/lib/mysql/ca.pem`
+/// as the CA certificate path, which is generated by the MySQL server automatically if
+/// no certificate is manually specified. Note that the path may vary based on the default `my.cnf`
+/// packaged with MySQL for your Linux distribution. Also note that unlike MySQL, MariaDB does *not*
+/// generate certificates automatically and they must always be passed in to enable TLS.
+///
+/// If `ssl-ca` is not specified or the file cannot be read, then an error is returned.
+/// `ssl-ca` implies `ssl-mode=VERIFY_CA` so you only actually need to specify the former
+/// but you may prefer having both to be more explicit.
+///
+/// If `ssl-mode=VERIFY_IDENTITY` is specified, in addition to checking the certificate as with
+/// `ssl-mode=VERIFY_CA`, the hostname in the connection string will be verified
+/// against the hostname in the server certificate, so they must be the same for the TLS
+/// upgrade to succeed. `ssl-ca` must still be specified.
 pub struct MySqlConnection {
-    pub(super) stream: BufStream<TcpStream>,
+    pub(super) stream: BufStream<MaybeTlsStream>,
 
     // Active capabilities of the client _&_ the server
     pub(super) capabilities: Capabilities,
@@ -171,6 +227,10 @@ impl MySqlConnection {
             client_capabilities |= Capabilities::CONNECT_WITH_DB;
         }
 
+        if cfg!(feature = "tls") {
+            client_capabilities |= Capabilities::SSL;
+        }
+
         self.capabilities =
             (client_capabilities & handshake.server_capabilities) | Capabilities::PROTOCOL_41;
 
@@ -197,9 +257,7 @@ impl MySqlConnection {
             }
         })
     }
-}
 
-impl MySqlConnection {
     pub(crate) fn handle_ok(&mut self) -> crate::Result<OkPacket> {
         let ok = OkPacket::decode(self.packet())?;
 
@@ -301,7 +359,14 @@ impl MySqlConnection {
     ) -> crate::Result<Box<[u8]>> {
         // https://mariadb.com/kb/en/caching_sha2_password-authentication-plugin/
 
-        // TODO: Handle SSL
+        if self.stream.is_tls() {
+            // If in a TLS stream, send the password directly in clear text
+            let mut clear_text = String::with_capacity(password.len() + 1);
+            clear_text.push_str(password);
+            clear_text.push('\0');
+
+            return Ok(clear_text.into_bytes().into_boxed_slice());
+        }
 
         // client sends a public key request
         self.send(&[public_key_request_id][..]).await?;
@@ -324,11 +389,17 @@ impl MySqlConnection {
 
 impl MySqlConnection {
     async fn new(url: &Url) -> crate::Result<Self> {
-        let stream = TcpStream::connect((url.host(), url.port(3306))).await?;
+        let stream = MaybeTlsStream::connect(url, 3306).await?;
+
+        let mut capabilities = Capabilities::empty();
+
+        if cfg!(feature = "tls") {
+            capabilities |= Capabilities::SSL;
+        }
 
         Ok(Self {
             stream: BufStream::new(stream),
-            capabilities: Capabilities::empty(),
+            capabilities,
             packet: Vec::with_capacity(8192),
             packet_len: 0,
             next_seq_no: 0,
@@ -372,10 +443,39 @@ impl MySqlConnection {
 
         Ok(())
     }
+
+    #[cfg(feature = "tls")]
+    async fn try_ssl(
+        &mut self,
+        url: &Url,
+        ca_file: Option<&str>,
+        invalid_hostnames: bool,
+    ) -> crate::Result<()> {
+        use crate::runtime::fs;
+        use async_native_tls::{Certificate, TlsConnector};
+
+        let mut connector = TlsConnector::new()
+            .danger_accept_invalid_certs(ca_file.is_none())
+            .danger_accept_invalid_hostnames(invalid_hostnames);
+
+        if let Some(ca_file) = ca_file {
+            let root_cert = fs::read(ca_file).await?;
+            connector = connector.add_root_certificate(Certificate::from_pem(&root_cert)?);
+        }
+
+        // send upgrade request and then immediately try TLS handshake
+        self.send(SslRequest {
+            client_collation: COLLATE_UTF8MB4_UNICODE_CI,
+            max_packet_size: MAX_PACKET_SIZE,
+        })
+        .await?;
+
+        self.stream.stream.upgrade(url, connector).await
+    }
 }
 
 impl MySqlConnection {
-    pub(super) async fn open(url: crate::Result<Url>) -> crate::Result<Self> {
+    pub(super) async fn establish(url: crate::Result<Url>) -> crate::Result<Self> {
         let url = url?;
         let mut self_ = Self::new(&url).await?;
 
@@ -383,7 +483,72 @@ impl MySqlConnection {
         // https://mariadb.com/kb/en/connection/
 
         // On connect, server immediately sends the handshake
-        let handshake = self_.receive_handshake(&url).await?;
+        let mut handshake = self_.receive_handshake(&url).await?;
+
+        let ca_file = url.get_param("ssl-ca");
+
+        let ssl_mode = url.get_param("ssl-mode").unwrap_or(
+            if ca_file.is_some() {
+                "VERIFY_CA"
+            } else {
+                "PREFERRED"
+            }
+            .into(),
+        );
+
+        let supports_ssl = handshake.server_capabilities.contains(Capabilities::SSL);
+
+        match &*ssl_mode {
+            "DISABLED" => (),
+
+            // don't try upgrade
+            #[cfg(feature = "tls")]
+            "PREFERRED" if !supports_ssl => {
+                log::warn!("server does not support TLS; using unencrypted connection")
+            }
+
+            // try to upgrade
+            #[cfg(feature = "tls")]
+            "PREFERRED" => {
+                if let Err(e) = self_.try_ssl(&url, None, true).await {
+                    log::warn!("TLS handshake failed, falling back to insecure: {}", e);
+                    // fallback, redo connection
+                    self_ = Self::new(&url).await?;
+                    handshake = self_.receive_handshake(&url).await?;
+                }
+            }
+
+            #[cfg(not(feature = "tls"))]
+            "PREFERRED" => log::info!("compiled without TLS, skipping upgrade"),
+
+            #[cfg(feature = "tls")]
+            "REQUIRED" if !supports_ssl => {
+                return Err(tls_err!("server does not support TLS").into())
+            }
+
+            #[cfg(feature = "tls")]
+            "REQUIRED" => self_.try_ssl(&url, None, true).await?,
+
+            #[cfg(feature = "tls")]
+            "VERIFY_CA" | "VERIFY_FULL" if ca_file.is_none() => {
+                return Err(
+                    tls_err!("`ssl-mode` of {:?} requires `ssl-ca` to be set", ssl_mode).into(),
+                )
+            }
+
+            #[cfg(feature = "tls")]
+            "VERIFY_CA" | "VERIFY_FULL" => {
+                self_
+                    .try_ssl(&url, ca_file.as_deref(), ssl_mode != "VERIFY_FULL")
+                    .await?
+            }
+
+            #[cfg(not(feature = "tls"))]
+            "REQUIRED" | "VERIFY_CA" | "VERIFY_FULL" => {
+                return Err(tls_err!("compiled without TLS").into())
+            }
+            _ => return Err(tls_err!("unknown `ssl-mode` value: {:?}", ssl_mode).into()),
+        }
 
         // Pre-generate an auth response by using the auth method in the [Handshake]
         let password = url.password().unwrap_or_default();
@@ -426,15 +591,30 @@ impl MySqlConnection {
     }
 }
 
-impl Connection for MySqlConnection {
-    fn open<T>(url: T) -> BoxFuture<'static, crate::Result<Self>>
+impl MySqlConnection {
+    #[deprecated(note = "please use 'connect' instead")]
+    pub fn open<T>(url: T) -> BoxFuture<'static, crate::Result<Self>>
     where
         T: TryInto<Url, Error = crate::Error>,
         Self: Sized,
     {
-        Box::pin(MySqlConnection::open(url.try_into()))
+        Box::pin(MySqlConnection::establish(url.try_into()))
     }
+}
 
+impl Connect for MySqlConnection {
+    type Connection = MySqlConnection;
+
+    fn connect<T>(url: T) -> BoxFuture<'static, crate::Result<MySqlConnection>>
+    where
+        T: TryInto<Url, Error = crate::Error>,
+        Self: Sized,
+    {
+        Box::pin(MySqlConnection::establish(url.try_into()))
+    }
+}
+
+impl Connection for MySqlConnection {
     fn close(self) -> BoxFuture<'static, crate::Result<()>> {
         Box::pin(self.close())
     }
