@@ -9,6 +9,8 @@ impl Database for Postgres {
     type Arguments = super::PgArguments;
 
     type Row = super::PgRow;
-}
 
-impl_into_arguments_for_database!(Postgres);
+    type TypeInfo = super::PgTypeInfo;
+
+    type TableId = u32;
+}
